@@ -1,10 +1,12 @@
 .First.lib <- function(lib=NULL, pkg=ks)
 {
   library.dynam("ks", pkg, lib)
-  cat("ks version 1.3.2 (2005) \n")
+  cat("ks version 1.3.3 (2005) \n")
 }  
 
 require(mvtnorm)
+require(misc3d)
+require(rgl)
 
 ###############################################################################
 # Basic operators and functions
